from ml_dash.schema import schema
from sanic_graphql import GraphQLView

from .file_handlers import get_path, remove_path, batch_get_path

from sanic import Sanic
from sanic_cors import CORS

app = Sanic(__name__)
# CORS(app)
CORS(app, resources={r"/*": {"origins": "*"}}, automatic_options=True)

# NOTE: disable for laziness. Should enable it in the future.
# @app.listener('before_server_start')
# def init_graphql(app, loop):
#     app.add_route(GraphQLView.as_view(schema=schema, executor=AsyncioExecutor(loop=loop)), '/graphql')

# new graphQL endpoints
app.add_route(GraphQLView.as_view(schema=schema, graphiql=True),
              '/graphql',
              methods=['GET', 'POST', 'FETCH', 'OPTIONS'])
app.add_route(GraphQLView.as_view(schema=schema, batch=True),
              '/graphql/batch',
              methods=['GET', 'POST', 'FETCH', 'OPTIONS'])

# # Serving static app
# app.add_route(get_path, '/*', methods=['GET', 'OPTIONS'])

# old RPC endpoints
app.add_route(get_path, '/files/', methods=['GET', 'OPTIONS'])
app.add_route(get_path, '/files/<file_path:path>', methods=['GET', 'OPTIONS'])
app.add_route(batch_get_path, '/batch-files', methods=['GET', 'OPTIONS'])
app.add_route(remove_path, '/files/<file_path:path>', methods=['DELETE'])

# note: currently disabled, file events API.
# from .file_events import file_events, setup_watch_queue
# app.add_route(file_events, '/file-events', methods=['GET', 'OPTIONS'])
# app.add_route(file_events, '/file-events/<file_path:path>', methods=['GET', 'OPTIONS'])
# app.listener('before_server_start')(setup_watch_queue)


def run(logdir=None, **kwargs):
    from . import config
    from termcolor import cprint

    if logdir:
        config.Args.logdir = logdir

    cprint("launched server with config:", "green")
    cprint("Args:", 'yellow')
    print(vars(config.Args))
    cprint("Sanic Server Args:", 'yellow')
    print(vars(config.ServerArgs))

    config.ServerArgs.update(**kwargs)
    app.run(**vars(config.ServerArgs))


if __name__ == "__main__":
    # see: https://sanic.readthedocs.io/en/latest/sanic/deploying.html
    # call this as `python -m ml_logger.main`
    run()



from graphene import relay, ObjectType, Float, Schema, AbstractType, List, String, Union, Field


# class Team(ObjectType):
#     class Meta:
#         interfaces = relay.Node,
#
#     name = String(description='string serialized data')
#     description = String(description='string serialized data')
#     users = List(lambda: User)
#     projects = List(lambda: Project)
#
#
# class Binder(ObjectType):
#     class Meta:
#         interfaces = relay.Node,
#
#     name = String(description="keys in the parameter file")
#     children = List("Binder", description="child binders")
#     files = List(lambda: FileAndDirectory, description="keys in the parameter file")
#     # date_created
#     # date_modified


class Directory(ObjectType):
    class Meta:
        interfaces = relay.Node,

    name = String(description='string serialized data')
    description = String(description='string serialized data')
    children = List(lambda: FileAndDirectory)


# File Types
class File(AbstractType):
    class Meta:
        interfaces = relay.Node,

    name = String(description='string serialized data')
    description = String(description='string serialized data')


class Parameters(ObjectType, File):
    pass


class Metrics(ObjectType, File):
    """this is just the file type."""
    pass


class Image(ObjectType, File):
    pass


class Video(ObjectType, File):
    pass


class TextFile(ObjectType, File):
    pass


class BinaryFile(ObjectType, File):
    pass


class JsonFile(ObjectType, File):
    pass


class YamlFile(ObjectType, File):
    pass


class FileAndDirectory(Union):
    class Meta:
        types = (Directory, File, Parameters, Metrics, TextFile, BinaryFile, JsonFile)


class ExperimentView(ObjectType):
    class Meta:
        interfaces = relay.Node,

    charts = List(lambda: Chart)


class Point(ObjectType):
    class Meta:
        interfaces = relay.Node,

    x = Float()
    y = Float()


class LineSeries(ObjectType):
    class Meta:
        interfaces = relay.Node,
        description = "simple line series"

    data = List(lambda: Point)
    label = String(description="label for the time series")


class StdSeries(ObjectType):
    class Meta:
        interfaces = relay.Node,
        description = "line series with standard deviation"

    data = List(lambda: Point)
    label = String(description="label for the series")


class QuantileSeries(ObjectType):
    class Meta:
        interfaces = relay.Node,
        description = "line series with quantile ticks"

    data = List(lambda: Point)
    label = String(description="label for the time series")


class Series(Union):
    class Meta:
        types = (LineSeries, StdSeries, QuantileSeries)


class LineChart(ObjectType):
    class Meta:
        interfaces = relay.Node,

    time_series = List(lambda: Series)

    title = String(description="title for the chart")
    x_label = String(description="label for the x axis")
    y_label = String(description="label for the y axis")
    x_low = String(description="lower range for the x axis")
    x_high = String(description="higher range for x axis")
    y_low = String(description="lower range for y axis")
    y_high = String(description="higher range for y axis")


class ParameterDomain(ObjectType):
    class Meta:
        interfaces = relay.Node,

    name = String()
    domain_low = Float()
    domain_high = Float()


class ParallelCoordinates(ObjectType):
    class Meta:
        interfaces = relay.Node,

    # todo: {key, value}
    data = List(ObjectType)
    domains = List(ParameterDomain)


class Chart(Union):
    """this is also a file type."""

    class Meta:
        types = LineChart, ParallelCoordinates
